//
//  TAdView
//
// This is the only header required for integrating into projects that will use the TAdView SDK.
//
//

/// *Required Frameworks*
///
/// These must be added to projects (via project's link build phase) that use the TAdView SDK.
///
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <EventKit/EventKit.h>
#import <EventKitUI/EventKitUI.h>
#import <MessageUI/MessageUI.h>
#import <CoreGraphics/CoreGraphics.h>
#import <ImageIO/ImageIO.h>
#import <CoreTelephony/CTTelephonyNetworkInfo.h>
#import <MediaPlayer/MediaPlayer.h>

@class TAdView;


/** Ad placement type.
 */
typedef enum
{
    /// Ad is placed in application content.
    TPlacementTypeInline = 0,
    
    /// Ad is placed over and in the way of application content.
    /// Generally used to place an ad between transtions in an application
    /// and consumes the entire screen.
    TPlacementTypeInterstitial
    
} TPlacementType;


/** Event log types.
 */
typedef enum
{
    TLogLevelNone = 0,
    TLogLevelError = 1,
    TLogLevelDebug = 2,
} TLogLevel;

/** Connection types.
 */
typedef enum
{
    TConnectionNone = 0,
    TConnectionCellular = 1,
    TConnectionWiFi = 2,
} TConnectionType;


/**
 
 */
@protocol TAdViewActivityListener <NSObject>
@optional
/** Sent before the ad opens a URL that invokes another application (ex: Safari or App Store).
 
 @param adView The TAdView instance sending the message.
 */
- (void)willLeaveApplication:(TAdView*)adView;

/** Sent when the ad will navigate to a clicked link.
 
 Not implementing this method behaves as if `YES` was returned.
 
 @param adView The TAdView instance sending the message.
 @param url The URL to open.
 @return `YES` Allow the SDK to open the link with UIApplication's openURL: or the internal browser.
 @return `NO` Ignore the request
 */
- (BOOL)TAdView:(TAdView*)adView shouldOpenURL:(NSURL*)url;


/** Sent when the close button is pressed by the user.
 
 This only occurs for the close button enabled with setCloseButton:afterDelay: or in the case of a
 interstitial richmedia ad that closes itself.  It will not be sent for richmedia close buttons that
 collapse expanded or resized ads.
 
 The common use case is for interstitial ads so the developer will know when to call closeInterstitial.
 
 @param adView The TAdView instance sending the message.
 */
- (void)closeButtonPressed:(TAdView*)adView;
@end

/**
 
 */
/**
 
 */
@protocol TAdViewInternalBrowserListener <NSObject>
@optional
/** Sent before the internal browser is opened.
 
 @param adView The TAdView instance sending the message.
 */
- (void)internalBrowserWillOpen:(TAdView*)adView;


/** Sent after the internal browser is opened.
 
 @param adView The TAdView instance sending the message.
 */
- (void)internalBrowserDidOpen:(TAdView*)adView;


/** Sent before the internal browser is closed.
 
 @param adView The TAdView instance sending the message.
 */
- (void)internalBrowserWillClose:(TAdView*)adView;


/** Sent after the internal browser is closed.
 
 @param adView The TAdView instance sending the message.
 */
- (void)internalBrowserDidClose:(TAdView*)adView;
@end

/**
 
 */
@protocol TAdViewLogListener <NSObject>
@optional
/** Sent when the ad view is about to log an event.
 
 Logging in the SDK is done with NSLog().  Implement and return `NO` to log to application specific
 log files.  This message will only be sent if the event type is equal to or higher than the TADView
 instance logLevel property.
 
 @param adView The TAdView instance sending the message.
 @param event The log event to log.
 @param type The event type.
 @return `YES` Log the event to NSLog().
 @return `NO` Omit logging the event to NSLog().
 */
- (BOOL)TAdView:(TAdView*)adView shouldLogEvent:(NSString*)event ofType:(TLogLevel)type;
@end

/**
 
 */
@protocol TAdViewRequestListener <NSObject>
@optional
/** Sent after an ad has been downloaded and rendered.
 
 @param adView The TAdView instance sending the message.
 */
- (void)didReceiveAd:(TAdView*)adView;


/** Sent if an error was encoutered while donloading or rendering an ad.
 
 @param adView The TAdView instance sending the message.
 @param error The error encountered while attempting to receive or render the ad.
 */
- (void)TAdView:(TAdView*)adView didFailToReceiveAdWithError:(NSError*)error;
@end

/**
 
 */
@protocol TAdViewRichMediaListener <NSObject>
@optional
/** Sent before the ad content is expanded in response to a richmedia expand event.
 
 The ad view itself is not expanded, instead a new window is displayed with the
 expanded ad content.
 
 @param adView The TAdView instance sending the message.
 */
- (void)willExpand:(TAdView*)adView;


/** Sent after the ad content is expanded in response to a richmedia expand event.
 
 The ad view itself is not expanded, instead a new window is displayed with the
 expanded ad content.
 
 @param adView The TAdView instance sending the message.
 */
- (void)didExpand:(TAdView*)adView;


/** Sent before the ad content is resized in response to a richmedia resize event.
 
 The ad view itself is not resized, instead a new window is displayed with the
 resized ad content.
 
 @param adView The TAdView instance sending the message.
 @param frame The frame relative to the window where the resized content is displayed.
 */
- (void)TAdView:(TAdView *)adView willResizeToFrame:(CGRect)frame;


/** Sent after the ad content is resized in response to a richmedia resize event.
 
 The ad view itself is not resized, instead a new window is displayed with the
 resized ad content.
 
 @param adView The TAdView instance sending the message.
 @param frame The frame relative to the window where the resized content is displayed.
 */
- (void)TAdView:(TAdView *)adView didResizeToFrame:(CGRect)frame;


/** Sent before ad content is collaped if expanded or resized.
 
 @param adView The TAdView instance sending the message.
 */
- (void)willCollapse:(TAdView*)adView;


/** Sent after ad content is collaped if expanded or resized.
 
 @param adView The TAdView instance sending the message.
 */
- (void)didCollapse:(TAdView*)adView;


/** Invoked when a rich media ad requests a video to be played.
 
 The default is to open video with MPMoviePlayerViewController.
 
 @param adView The TAdView instance sending the message.
 @param videoURL The URL string of the video to play.
 @return `NO` Do not open the URL and play the video.
 @return `YES` Open video with MPMoviePlayerViewController.
 */
- (BOOL)TAdView:(TAdView*)adView shouldPlayVideo:(NSString*)videoURL;


/** Invoked when JS error fired while running script.
 
 By default SDK will remove ad
 
 @param adView The TAdView instance sending the message.
 @param error  The description of error.
 @return `NO` Allow the developer to handle the error.
 @return `YES` Indicate the error should be handled by the application.
 */
- (BOOL)shouldResetAdOnFailedToRunJs:(TAdView*)adView error:(NSString *)error;
@end

/** Protocal for interaction with the TAdView.
 
 The entire protocol is optional.  Some messages override default behavior and some are required
 to get full support for MRAID 2 ad content (saving calendar entries or pictures).
 
 All messages are guaranteed to occur on the main thread.  If any long running tasks are needed
 in reponse to any of the sent messages then they should be executed in a background thread to
 prevent and UI delays for the user.
 */
@protocol TAdViewDelegate  <TAdViewActivityListener,TAdViewInternalBrowserListener, TAdViewLogListener, TAdViewRequestListener,
    TAdViewRichMediaListener>
@optional

/** Allow developers to set the main window.
 
 Use it if the application supports external displays.
 
 @return The window object.
 */

- (UIWindow *)TGetMainWindow;
@end


/** Renders text, image and richmedia ads.
 
 TODO: Include more information here and possibly a quick sample.
 
 The -ObjC linker option is required when including the TAdView SDK.
 
 */
@interface TAdView : UIView


 /** Returns the TAdView SDK's version.
 */
+ (NSString*)version;


/** Unregisters the protocol class used to intercept the MRAID bridge request
 from rich media ads.
 
 Note: The registered NSURLProtocol class used by the SDK only intercepts requests
 for "mraid.js" from a UIWebView.
 
 @see [NSURLProtocol](https://developer.apple.com/library/mac/#documentation/Cocoa/Reference/Foundation/Classes/NSURLProtocol_Class/Reference/Reference.html) for any possible impact to the application.
 */
+ (void)unregisterProtocolClass;


///---------------------------------------------------------------------------------------
/// @name Initialization
///---------------------------------------------------------------------------------------

/** Initilizes an inline instance of the ad view.
 
 The view can be added to other views as with any other UIView object.  The frame is
 used to determine the size of the ad in the requested to the ad server.  If not known
 at initialization time, ensure that the view's frame is set prior to calling update.
 
 @param frame The area to place the view.
 */
- (id)initWithFrame:(CGRect)frame withDelegate:(id<TAdViewDelegate>)dlgt;


/** Initializes an interstital instance of the ad view.
 
 The view is NOT intended to be used inline with other content or added to
 other views.  Instead use the interstitial methods to show and close the
 full screen view.
 
 @see showInterstitial
 @see showInterstitialWithDuration:
 @see closeInterstitial
 */
- (id)initInterstitial:(id<TAdViewDelegate>)dlgt;


/** Returns the placement type for the instance.
 
 This is set based on how the instance was initialized.
 
 @see initWithFrame:
 @see initInterstitial
 */
@property (nonatomic, readonly) TPlacementType placementType;

/**
 Specifies the ad type value
 */
@property (nonatomic, assign) NSInteger adType;


/**
 Specifies the app id
 */
@property (nonatomic, copy) NSString* applicationId;

/**
 Specifies IP address
 */
@property (nonatomic, copy) NSString* IPAddress;

/**
 Specifies the test string to supply with the ad
 */
@property (nonatomic, copy) NSString* test;


/**
 Specifies whether toast message is OK to be shown by AdView.
 Default value is YES.
 */
@property (nonatomic, assign) BOOL showToasts;

/**
 Specifies the ad content to use. For tests
 */
@property (nonatomic, copy) NSString* testAdContent;


///---------------------------------------------------------------------------------------
/// @name Optional configuration
///---------------------------------------------------------------------------------------

// Set the server and additional parameters as required.
// These are only needed for advanced usages.

/** Specifies the URL of the ad server.
 */
@property (nonatomic, copy) NSString* adNetworkURL;


/** Allows setting extra server parameters.
 
 The SDK will set various parameters based on configuration and other options.
 
 @warning All parameter key and values must be NSString instances.
 */
@property (nonatomic, readonly) NSMutableDictionary* adRequestParameters;


/** Set to enable the use of the internal browser for opening ad content.  Defaults to `NO`.
 
 @see isInternalBrowserOpen
 @see [TAdViewDelegate TAdViewInternalBrowserWillOpen:]
 @see [TAdViewDelegate TAdViewInternalBrowserDidOpen:]
 @see [TAdViewDelegate TAdViewInternalBrowserWillClose:]
 @see [TAdViewDelegate TAdViewInternalBrowserDidClose:]
 */
@property (nonatomic, assign) BOOL useInternalBrowser;


/** Returns the status of the internal browser.
 
 @see useInternalBrowser
 */
@property (nonatomic, readonly) BOOL isInternalBrowserOpen;


/** Sets the TAdViewDelegate delegate receiever for the ad view.
 
 @warning Proper reference management practices should be observed when using delegates.
 @warning Ensure that the delegate is set to nil prior to releasing the ad view's instance.
 */
@property (nonatomic, weak) id<TAdViewDelegate> delegate;


///---------------------------------------------------------------------------------------
/// @name Updating and resetting ad content
///---------------------------------------------------------------------------------------

/** Issues an update request.  The request is deferred until user action is completed.
 
 @see update:
 */
- (void)update;


/** Issues an update.
 
 Resets any interval update from updateWithTimeInterval:.
 
 The update will be deferred if the user is interacting with the ad instance.  This can 
 include the internal browser being open or a rich media ad in an expanded or resized state.
 If deferred the update will resume when the interaction is completed.  Specifying YES to
 the force property will close any user interaction and perform the update immediately.
 
 If [TAdViewDelegate TAdViewSupportsCalendar:] is implemented by the delegate then
 this message will determine if a request to the user for access to the calendar is needed.
 If so, the instance will request access from the user.  Developers desiring to control when
 this request occurs can do so prior to calling update.  Refer to iOS EKEventStore
 documentation for more information.

 @param force Set to `YES` to force an update regardless of current ad status/interaction.
 */
- (void)update:(BOOL)force;


/** Issues an update request that will be deferred if the ad is currently being interacted with.
 Will automatically update every interval seconds (can vary depending on user interaction).
 
 If [TAdViewDelegate TAdViewSupportsCalendar:] is implemented by the delegate then
 this message will determine if a request to the user for access to the calendar is needed.
 If so, the instance will request access from the user.  Developers desiring to control when
 this request occurs can do so prior to calling update.  Refer to iOS EKEventStore
 documentation for more information.
 
 @warning Ensure [TAdView reset] is invoked when the instance will no longer be used or
 is being removed from the view stack.  This will prevent the main NSRunLoop from retaining
 the TAdView instance after its intended release.
 
 @see reset
 
 @param interval The delay between requesting updates after the initial update.
 */
- (void)updateWithTimeInterval:(NSTimeInterval)interval;


/** Restates the instance to its default state.
 
 -Stops updates and cancels the update interval.
 -Collapses any expanded or resized richmedia ads.
 -Closes interstitial.
 -Closes internal ad browser.
 
 Should be sent before releasing the instance if another object may be retaining it 
 such as a superview or list.  This allows the application to suspend ad updating 
 and interaction activities to allow other application activitis to occur.  After
 responding to other activities update or updateWithTimeInterval: can be sent again
 to resume ad updates.
 
 @warning If the project is using ARC (automatic reference counting) this MUST be called
 to cancel internal timers.  If not the main NSRunLoop will retain a reference to the 
 TAdView instance and continue invoking its timers.
 @warning Does not reset the delegate.
 */
- (void)reset;


/** Removes any displayed ad content and any associated state.
 
 -Collapses any expanded or resized richmedia ads.
 -Closes interstitial.
 
 Cancels any deferred update.
 
 Unlike reset, it does not reset the instance to it's default state.
 
 @see update:
 */
- (void)removeContent;


///---------------------------------------------------------------------------------------
/// @name Handling of Features.
///---------------------------------------------------------------------------------------

/** Disable support calendar event.
 
  If create calendar event should be reported as a NOT supported feature.
 */
- (void)forceStopCalendarSupport;


/** Disable support image event.
 
  If store image should be reported as a NOT supported feature,.
 */
- (void)forceStopImageSaveSupport;


/** Disable support sms event.
 
  If sending SMS should be reported as a NOT supported feature,.
 */
- (void)forceStopSmsSupport;

/** Disable support phone dialer event.
 
 If opening phone dialer should be reported as a NOT supported feature.
 */
- (void)forceStopTelSupport;


///---------------------------------------------------------------------------------------
/// @name Controlling interstitial presentation
///---------------------------------------------------------------------------------------

/** Shows and closes the interstitial view.

 Can only be used if the instance was initialized with initInterstitial.
 */
- (void)showInterstitial;


/** Shows the interstitial and automatically closes after the specified duration.
 
 Can only be used if the instance was initialized with initInterstitial.
 
 @param duration The amount of time to display the interstitial before closing it.
 */
- (void)showInterstitialWithDuration:(NSTimeInterval)duration;


/** Closes the interstitial.
 
 */
- (void)closeInterstitial;

/**
 Checks the placement type if the ad is interstitial
 */
- (BOOL)isInterstitial;


///---------------------------------------------------------------------------------------
/// @name Close button support
///---------------------------------------------------------------------------------------

/** Shows a close button after the specified delay after the ad is rendered.
 
 This can be used for both inline/banner/custom and interstitial ads.  For most cases
 this should not be required since banner ads don't usually have a need for a close 
 button and richmedia ads that expand or resize will offer their own close button.
 
 This SHOULD be used for interstitial ads that are known to not be richmedia as they
 will not have a built in close button.
 
 The setting applies for all subsequent updates.  The button can be customized using the 
 TAdViewCustomCloseButton: delegate method.
 
 @warning Ensure [TAdView reset] is invoked when the instance will no longer be used or
 is being removed from the view stack.  This will prevent the main NSRunLoop from retaining 
 the TAdView instance after its intended release.
 
 @see reset
 
 @param showCloseButton Set to `YES` to display the close button after rendering ads.
 @param delay The time to delay showing the close button after rendering the ad.  A
 value of 0 will show the button immediately.
 */
- (void)showCloseButton:(BOOL)showCloseButton afterDelay:(NSTimeInterval)delay;

/** Close button delay.
 It can be modified to change the  delay of close button appearance
 */
@property (nonatomic, assign) NSTimeInterval closeButtonDelay;

///---------------------------------------------------------------------------------------
/// @name Ad containers
///---------------------------------------------------------------------------------------

/** Web view ad container.
 
 This can be modified to change aspects of the web view such as background color, etc..
 
 @warning Do not attempt to move or affect the relationship of the view within its superview.
 @warning Do not attempt to change the delegate or affect how content is rendered as this may
 interfere with richmedia ads.
 */
@property (nonatomic, strong) UIWebView* webView;

/** Richmedia expand view container.
 
 This view is the container used to hold the view to be expanded.  For richmedia ads that request
 an expand this will contain the UIWebView.  This doesn't include richmedia ads that expand with
 another creative (two part expand).
 
 @warning Do not attempt to move or affect the relationship of the view within its superview.
 */
@property (nonatomic, readonly) UIView* expandView;

/** Richmedia resize view container.
 
 This view is the container used to render resized richmedia ad it requests to resize.
 
 @warning Do not attempt to move or affect the relationship of the view within its superview.
 */
@property (nonatomic, readonly) UIView* resizeView;


///---------------------------------------------------------------------------------------
/// @name Test content, logging and debugging
///---------------------------------------------------------------------------------------


/** Specifies the log level.  All logging is via NSLog().
 
 @see [TAdViewDelegate TAdView:shouldLogEvent:ofType:]
 */
@property (nonatomic, assign) TLogLevel logLevel;

/**
 Returns the device ID
 */
+ (NSString*) getDeviceId;


/**
 Used by webView to post the log events
 */
- (void)logEvent:(NSString*)event ofType:(TLogLevel)type func:(const char*)func line:(int)line;

/**
 Used for determine connection type
 */
- (int) getConnectionType;

/**
 Used for determine user agent
 */
- (NSString *) getUserAgent;

@end
